@@ -26,6 +26,7 @@ let () =
     | 5, 0 ->
         "414"
         (* Ast_500 aliases Ast_414, since the AST hasn't changed between those two *)
+    | 5, 1 -> "501"
     | _ ->
-        Printf.eprintf "Unkown OCaml version %s\n" ocaml_version_str;
+        Printf.eprintf "Unknown OCaml version %s\n" ocaml_version_str;
         exit 1)
