@@ -1,7 +1,7 @@
 open! Import
 open Ast_builder.Default
-module Buffer = Caml.Buffer
-module Format = Caml.Format
+module Buffer = Stdlib.Buffer
+module Format = Stdlib.Format
 
 let lident x = Longident.Lident x
 
@@ -97,7 +97,7 @@ exception Type_is_recursive
 
 class type_is_recursive rec_flag tds =
   object (self)
-    inherit Ast_traverse.iter as super
+    inherit Ast_traverse0.iter as super
     val type_names : string list = List.map tds ~f:(fun td -> td.ptype_name.txt)
     method return_true () = raise_notrace Type_is_recursive
 
@@ -185,7 +185,7 @@ let attributes_errors =
 
 let collect_attributes_errors =
   object
-    inherit [Location.Error.t list] Ast_traverse.fold
+    inherit [Location.Error.t list] Ast_traverse0.fold
     method! attribute a acc = attributes_errors [ a ] @ acc
   end
 
@@ -196,7 +196,7 @@ let assert_no_attributes l =
 
 let assert_no_attributes_in =
   object
-    inherit Ast_traverse.iter
+    inherit Ast_traverse0.iter
     method! attribute a = assert_no_attributes [ a ]
   end
 
@@ -253,3 +253,25 @@ let mk_named_sig ~loc ~sg_name ~handle_polymorphic_variant = function
                   (pmty_ident ~loc (Located.lident mty ~loc))
                   [ Pwith_typesubst (Located.lident ~loc "t", for_subst) ]))
   | _ -> None
+
+let exn_to_loc_error exn =
+  match Location.Error.of_exn exn with Some error -> error | None -> raise exn
+
+module With_errors = struct
+  type 'a t = 'a * Location.Error.t list
+
+  let return e = (e, [])
+
+  let ( >>= ) (x, errors1) f =
+    let y, errors2 = f x in
+    (y, errors1 @ errors2)
+
+  let ( >>| ) (x, errors) f = (f x, errors)
+
+  let of_result result ~default =
+    match result with
+    | Ok x -> (x, [])
+    | Error errors -> (default, NonEmptyList.to_list errors)
+
+  let combine_errors list = (List.map list ~f:fst, List.concat_map list ~f:snd)
+end
