@@ -10,7 +10,7 @@ let keep_w32_intf = ref false
 
 let () =
   let keep_w32_spec =
-    Caml.Arg.Symbol
+    Stdlib.Arg.Symbol
       ( [ "impl"; "intf"; "both" ],
         function
         | "impl" -> keep_w32_impl := true
@@ -21,7 +21,7 @@ let () =
         | _ -> assert false )
   in
   let conv_w32_spec =
-    Caml.Arg.Symbol
+    Stdlib.Arg.Symbol
       ( [ "code"; "attribute" ],
         function
         | "code" -> do_insert_unused_warning_attribute := false
@@ -44,7 +44,7 @@ let keep_w60_intf = ref false
 
 let () =
   let keep_w60_spec =
-    Caml.Arg.Symbol
+    Stdlib.Arg.Symbol
       ( [ "impl"; "intf"; "both" ],
         function
         | "impl" -> keep_w60_impl := true
@@ -59,6 +59,14 @@ let () =
 
 let keep_w60_impl () = !keep_w60_impl || Driver.pretty ()
 let keep_w60_intf () = !keep_w60_intf || Driver.pretty ()
+let allow_unused_code_warnings = ref Options.default_allow_unused_code_warnings
+
+let () =
+  Driver.add_arg "-unused-code-warnings"
+    (Bool (( := ) allow_unused_code_warnings))
+    ~doc:"_ Allow ppx derivers to enable unused code warnings"
+
+let allow_unused_code_warnings () = !allow_unused_code_warnings
 
 module Args = struct
   include (
@@ -133,6 +141,8 @@ type parsed_args =
   | Args of (string * expression) list
   | Unknown_syntax of Location.t * string
 
+type 'item derived_code = { items : 'item list; unused_code_warnings : bool }
+
 module Generator = struct
   type deriver = t
 
@@ -141,26 +151,29 @@ module Generator = struct
         spec : ('c, 'a) Args.t;
         gen : ctxt:Expansion_context.Deriver.t -> 'b -> 'c;
         arg_names : String.Set.t;
-        attributes : Attribute.packed list;
         deps : deriver list;
+        unused_code_warnings : bool;
       }
         -> ('a, 'b) t
 
   let deps (T t) = t.deps
 
   module V2 = struct
-    let make ?(attributes = []) ?(deps = []) spec gen =
+    let make ?attributes:(_ = []) ?(deps = []) ?(unused_code_warnings = false)
+        spec gen =
       let arg_names = String.Set.of_list (Args.names spec) in
-      T { spec; gen; arg_names; attributes; deps }
+      T { spec; gen; arg_names; deps; unused_code_warnings }
 
-    let make_noarg ?attributes ?deps gen = make ?attributes ?deps Args.empty gen
+    let make_noarg ?attributes ?deps ?unused_code_warnings gen =
+      make ?attributes ?deps ?unused_code_warnings Args.empty gen
   end
 
-  let make ?attributes ?deps spec gen =
-    V2.make ?attributes ?deps spec
+  let make ?attributes ?deps ?unused_code_warnings spec gen =
+    V2.make ?attributes ?deps ?unused_code_warnings spec
       (Expansion_context.Deriver.with_loc_and_path gen)
 
-  let make_noarg ?attributes ?deps gen = make ?attributes ?deps Args.empty gen
+  let make_noarg ?attributes ?deps ?unused_code_warnings gen =
+    make ?attributes ?deps ?unused_code_warnings Args.empty gen
 
   let merge_accepted_args l =
     let rec loop acc = function
@@ -215,8 +228,11 @@ module Generator = struct
   let apply_all ~ctxt entry (name, generators, args) =
     let open Result in
     check_arguments name.txt generators args >>| fun () ->
-    List.concat_map generators ~f:(fun t ->
-        apply t ~name:name.txt ~ctxt entry args)
+    List.map generators ~f:(fun (T t) ->
+        {
+          items = apply (T t) ~name:name.txt ~ctxt entry args;
+          unused_code_warnings = t.unused_code_warnings;
+        })
 
   let apply_all ~ctxt entry generators ext_to_item =
     let l = List.map generators ~f:(apply_all ~ctxt entry) in
@@ -227,7 +243,7 @@ module Generator = struct
       List.concat lerr
       |> List.map ~f:(fun err -> ext_to_item ~loc:Location.none err [])
     in
-    List.concat l1 @ lerr
+    List.concat l1 @ [ { items = lerr; unused_code_warnings = false } ]
 end
 
 module Deriver = struct
@@ -602,6 +618,7 @@ let inline_doc_attr =
     attr_loc = loc;
   }
 
+(* wrap a structure in extra attributes *)
 let wrap_str ~loc ~hide st =
   let include_infos = include_infos ~loc (pmod_structure ~loc st) in
   let pincl_attributes =
@@ -610,17 +627,21 @@ let wrap_str ~loc ~hide st =
   in
   [ pstr_include ~loc { include_infos with pincl_attributes } ]
 
-let wrap_str ~loc ~hide st =
+(* decide what to wrap a structure in, then call above [wrap_str] *)
+let wrap_str ~loc ~hide ~unused_code_warnings st =
   let loc = { loc with loc_ghost = true } in
+  let unused_code_warnings =
+    unused_code_warnings && allow_unused_code_warnings ()
+  in
   let warnings, st =
-    if keep_w32_impl () then ([], st)
+    if keep_w32_impl () || unused_code_warnings then ([], st)
     else if not !do_insert_unused_warning_attribute then
       ([], Ignore_unused_warning.add_dummy_user_for_values#structure st)
     else ([ 32 ], st)
   in
   let warnings, st =
     if
-      keep_w60_impl ()
+      keep_w60_impl () || unused_code_warnings
       || not (Ignore_unused_warning.binds_module_names#structure st false)
     then (warnings, st)
     else (60 :: warnings, st)
@@ -631,6 +652,13 @@ let wrap_str ~loc ~hide st =
   in
   if wrap then wrap_str ~loc ~hide st else st
 
+(* wrap blocks that share [unused_code_warnings], using above [wrap_str] above *)
+let wrap_str ~loc ~hide list =
+  List.concat_map list ~f:(fun { items; unused_code_warnings } ->
+      if List.is_empty items then []
+      else wrap_str ~loc ~hide ~unused_code_warnings items)
+
+(* wrap a signature in extra attributes *)
 let wrap_sig ~loc ~hide st =
   let include_infos = include_infos ~loc (pmty_signature ~loc st) in
   let pincl_attributes =
@@ -639,13 +667,20 @@ let wrap_sig ~loc ~hide st =
   in
   [ psig_include ~loc { include_infos with pincl_attributes } ]
 
-let wrap_sig ~loc ~hide sg =
+(* decide what to wrap a signature in, then call above [wrap_sig] *)
+let wrap_sig ~loc ~hide ~unused_code_warnings sg =
   let loc = { loc with loc_ghost = true } in
-  let warnings = if keep_w32_intf () then [] else [ 32 ] in
+  let unused_code_warnings =
+    unused_code_warnings && allow_unused_code_warnings ()
+  in
+  let warnings =
+    if keep_w32_intf () || unused_code_warnings then [] else [ 32 ]
+  in
   let warnings =
     if
       keep_w60_intf ()
-      || not (Ignore_unused_warning.binds_module_names#signature sg false)
+      || (not (Ignore_unused_warning.binds_module_names#signature sg false))
+      || unused_code_warnings
     then warnings
     else 60 :: warnings
   in
@@ -655,29 +690,12 @@ let wrap_sig ~loc ~hide sg =
   in
   if wrap then wrap_sig ~loc ~hide sg else sg
 
-(* +-----------------------------------------------------------------+
-   | Remove attributes used by syntax extensions                     |
-   +-----------------------------------------------------------------+ *)
-(*
-let remove generators =
-  let attributes =
-    List.concat_map generators ~f:(fun (_, actual_generators, _) ->
-      List.concat_map actual_generators ~f:(fun (Generator.T g) -> g.attributes))
-  in
-  object
-    inherit Ast_traverse.map
+(* wrap blocks that share [unused_code_warnings], using above [wrap_sig] above *)
+let wrap_sig ~loc ~hide list =
+  List.concat_map list ~f:(fun { items; unused_code_warnings } ->
+      if List.is_empty items then []
+      else wrap_sig ~loc ~hide ~unused_code_warnings items)
 
-    (* Don't recurse through attributes and extensions *)
-    method! attribute x = x
-    method! extension x = x
-
-    method! label_declaration ld =
-      Attribute.remove_seen Attribute.Context.label_declaration attributes ld
-
-    method! constructor_declaration cd =
-      Attribute.remove_seen Attribute.Context.constructor_declaration attributes cd
-  end
-*)
 (* +-----------------------------------------------------------------+
    | Main expansion                                                  |
    +-----------------------------------------------------------------+ *)
@@ -700,6 +718,17 @@ let types_used_by_deriving (tds : type_declaration list) : structure_item list =
 let merge_generators field l =
   List.filter_map l ~f:(fun x -> x) |> List.concat |> Deriver.resolve_all field
 
+(* This function merges ['a derived] if they have the same [unused_code_warnings]. This
+   reduces the number of times we add [include struct ... end] to disable warnings. *)
+let merge_derived lists =
+  List.fold_right lists ~init:[] ~f:(fun derived acc ->
+      match acc with
+      | other :: others
+        when Bool.equal derived.unused_code_warnings other.unused_code_warnings
+        ->
+          { other with items = derived.items @ other.items } :: others
+      | _ -> derived :: acc)
+
 let expand_str_type_decls ~ctxt rec_flag tds values =
   let generators, l_err = merge_generators Deriver.Field.str_type_decl values in
   let l_err =
@@ -711,9 +740,10 @@ let expand_str_type_decls ~ctxt rec_flag tds values =
   (* TODO: instead of disabling the unused warning for types themselves, we
      should add a tag [@@unused]. *)
   let generated =
-    types_used_by_deriving tds @ l_err
-    @ Generator.apply_all ~ctxt (rec_flag, tds) generators
-        Ast_builder.Default.pstr_extension
+    { items = types_used_by_deriving tds @ l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt (rec_flag, tds) generators
+         Ast_builder.Default.pstr_extension
+    |> merge_derived
   in
   wrap_str
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
@@ -729,9 +759,10 @@ let expand_sig_type_decls ~ctxt rec_flag tds values =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt (rec_flag, tds) generators
-        Ast_builder.Default.psig_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt (rec_flag, tds) generators
+         Ast_builder.Default.psig_extension
+    |> merge_derived
   in
   wrap_sig
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
@@ -749,11 +780,11 @@ let expand_str_module_type_decl ~ctxt mtd generators =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt mtd generators
-        Ast_builder.Default.pstr_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt mtd generators
+         Ast_builder.Default.pstr_extension
+    |> merge_derived
   in
-
   wrap_str
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
     ~hide:(not @@ Expansion_context.Deriver.inline ctxt)
@@ -770,9 +801,10 @@ let expand_sig_module_type_decl ~ctxt mtd generators =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt mtd generators
-        Ast_builder.Default.psig_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt mtd generators
+         Ast_builder.Default.psig_extension
+    |> merge_derived
   in
   wrap_sig
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
@@ -790,8 +822,10 @@ let expand_str_exception ~ctxt ec generators =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt ec generators Ast_builder.Default.pstr_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt ec generators
+         Ast_builder.Default.pstr_extension
+    |> merge_derived
   in
   wrap_str
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
@@ -809,8 +843,10 @@ let expand_sig_exception ~ctxt ec generators =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt ec generators Ast_builder.Default.psig_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt ec generators
+         Ast_builder.Default.psig_extension
+    |> merge_derived
   in
   wrap_sig
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
@@ -828,8 +864,10 @@ let expand_str_type_ext ~ctxt te generators =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt te generators Ast_builder.Default.pstr_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt te generators
+         Ast_builder.Default.pstr_extension
+    |> merge_derived
   in
   wrap_str
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
@@ -847,8 +885,10 @@ let expand_sig_type_ext ~ctxt te generators =
       l_err
   in
   let generated =
-    l_err
-    @ Generator.apply_all ~ctxt te generators Ast_builder.Default.psig_extension
+    { items = l_err; unused_code_warnings = false }
+    :: Generator.apply_all ~ctxt te generators
+         Ast_builder.Default.psig_extension
+    |> merge_derived
   in
   wrap_sig
     ~loc:(Expansion_context.Deriver.derived_item_loc ctxt)
