@@ -1,9 +1,11 @@
 (*$ open Ppxlib_cinaps_helpers $*)
 open Import
 open Utils
-module Arg = Caml.Arg
+open Common
+open With_errors
+module Arg = Stdlib.Arg
 
-let exe_name = Caml.Filename.basename Caml.Sys.executable_name
+let exe_name = Stdlib.Filename.basename Stdlib.Sys.executable_name
 let args = ref []
 let add_arg key spec ~doc = args := (key, spec, doc) :: !args
 let loc_fname = ref None
@@ -23,6 +25,31 @@ let styler = ref None
 let output_metadata_filename = ref None
 let corrected_suffix = ref ".ppx-corrected"
 
+let ghost =
+  object
+    inherit Ast_traverse.map
+    method! location loc = { loc with loc_ghost = true }
+  end
+
+let chop_prefix ~prefix x =
+  if String.is_prefix ~prefix x then
+    Some (String.drop_prefix x (String.length prefix))
+  else None
+
+let get_default_path (loc : Location.t) =
+  let fname = loc.loc_start.pos_fname in
+  match chop_prefix ~prefix:"./" fname with
+  | Some fname -> fname
+  | None -> fname
+
+let get_default_path_str : structure -> string = function
+  | [] -> ""
+  | { pstr_loc = loc; _ } :: _ -> get_default_path loc
+
+let get_default_path_sig : signature -> string = function
+  | [] -> ""
+  | { psig_loc = loc; _ } :: _ -> get_default_path loc
+
 module Lint_error = struct
   type t = Location.t * string
 
@@ -69,12 +96,16 @@ module Instrument = struct
 
   type t = {
     transformation :
-      Expansion_context.Base.t -> Parsetree.structure -> Parsetree.structure;
+      Expansion_context.Base.t ->
+      Parsetree.structure ->
+      Parsetree.structure With_errors.t;
     position : pos;
   }
 
   module V2 = struct
-    let make transformation ~position = { transformation; position }
+    let make transformation ~position =
+      let transformation ctx st = return (transformation ctx st) in
+      { transformation; position }
   end
 
   let make transformation ~position =
@@ -87,10 +118,14 @@ module Transform = struct
     name : string;
     aliases : string list;
     impl :
-      (Expansion_context.Base.t -> Parsetree.structure -> Parsetree.structure)
+      (Expansion_context.Base.t ->
+      Parsetree.structure ->
+      Parsetree.structure With_errors.t)
       option;
     intf :
-      (Expansion_context.Base.t -> Parsetree.signature -> Parsetree.signature)
+      (Expansion_context.Base.t ->
+      Parsetree.signature ->
+      Parsetree.signature With_errors.t)
       option;
     lint_impl :
       (Expansion_context.Base.t -> Parsetree.structure -> Lint_error.t list)
@@ -99,10 +134,14 @@ module Transform = struct
       (Expansion_context.Base.t -> Parsetree.signature -> Lint_error.t list)
       option;
     preprocess_impl :
-      (Expansion_context.Base.t -> Parsetree.structure -> Parsetree.structure)
+      (Expansion_context.Base.t ->
+      Parsetree.structure ->
+      Parsetree.structure With_errors.t)
       option;
     preprocess_intf :
-      (Expansion_context.Base.t -> Parsetree.signature -> Parsetree.signature)
+      (Expansion_context.Base.t ->
+      Parsetree.signature ->
+      Parsetree.signature With_errors.t)
       option;
     enclose_impl :
       (Expansion_context.Base.t ->
@@ -133,7 +172,7 @@ module Transform = struct
       ?impl ?intf ?lint_impl ?lint_intf ?preprocess_impl ?preprocess_intf
       ?instrument ?(aliases = []) name =
     let rules = List.map extensions ~f:Context_free.Rule.extension @ rules in
-    let caller_id = Caller_id.get ~skip:[ Caml.__FILE__ ] in
+    let caller_id = Caller_id.get ~skip:[ Stdlib.__FILE__ ] in
     (match List.filter !all ~f:(fun ct -> has_name ct name) with
     | [] -> ()
     | ct :: _ ->
@@ -142,6 +181,14 @@ module Transform = struct
         Printf.eprintf "  - first time was at %a\n" print_caller_id
           ct.registered_at;
         Printf.eprintf "  - second time is at %a\n" print_caller_id caller_id);
+    let impl = Option.map impl ~f:(fun f ctx ast -> return (f ctx ast)) in
+    let intf = Option.map intf ~f:(fun f ctx ast -> return (f ctx ast)) in
+    let preprocess_impl =
+      Option.map preprocess_impl ~f:(fun f ctx ast -> return (f ctx ast))
+    in
+    let preprocess_intf =
+      Option.map preprocess_intf ~f:(fun f ctx ast -> return (f ctx ast))
+    in
     let ct =
       {
         name;
@@ -171,12 +218,12 @@ module Transform = struct
         let last = get_loc (last x l) in
         Some { first with loc_end = last.loc_end }
 
-  let merge_into_generic_mappers t ~hook ~expect_mismatch_handler ~tool_name
-      ~input_name =
+  let merge_into_generic_mappers t ~embed_errors ~hook ~expect_mismatch_handler
+      ~tool_name ~input_name =
     let { rules; enclose_impl; enclose_intf; impl; intf; _ } = t in
     let map =
       new Context_free.map_top_down
-        rules ~generated_code_hook:hook ~expect_mismatch_handler
+        rules ~embed_errors ~generated_code_hook:hook ~expect_mismatch_handler
     in
     let gen_header_and_footer context whole_loc f =
       let header, footer = f whole_loc in
@@ -212,52 +259,48 @@ module Transform = struct
       match input_name with Some input_name -> input_name | None -> "_none_"
     in
     let map_impl ctxt st_with_attrs =
-      let st =
-        let attrs, st =
-          List.split_while st_with_attrs ~f:(function
-            | { pstr_desc = Pstr_attribute _; _ } -> true
-            | _ -> false)
-        in
-        let file_path = File_path.get_default_path_str st in
-        let base_ctxt =
-          Expansion_context.Base.top_level ~tool_name ~file_path ~input_name
-        in
-        let header, footer =
-          match enclose_impl with
-          | None -> ([], [])
-          | Some f ->
-              let whole_loc =
-                loc_of_list st ~get_loc:(fun st -> st.Parsetree.pstr_loc)
-              in
-              gen_header_and_footer Structure_item whole_loc (f base_ctxt)
-        in
-        map#structure base_ctxt (List.concat [ attrs; header; st; footer ])
+      let attrs, st =
+        List.split_while st_with_attrs ~f:(function
+          | { pstr_desc = Pstr_attribute _; _ } -> true
+          | _ -> false)
       in
-      match impl with None -> st | Some f -> f ctxt st
+      let file_path = get_default_path_str st in
+      let base_ctxt =
+        Expansion_context.Base.top_level ~tool_name ~file_path ~input_name
+      in
+      let header, footer =
+        match enclose_impl with
+        | None -> ([], [])
+        | Some f ->
+            let whole_loc =
+              loc_of_list st ~get_loc:(fun st -> st.Parsetree.pstr_loc)
+            in
+            gen_header_and_footer Structure_item whole_loc (f base_ctxt)
+      in
+      map#structure base_ctxt (List.concat [ attrs; header; st; footer ])
+      >>= fun st -> match impl with None -> return st | Some f -> f ctxt st
     in
     let map_intf ctxt sg_with_attrs =
-      let sg =
-        let attrs, sg =
-          List.split_while sg_with_attrs ~f:(function
-            | { psig_desc = Psig_attribute _; _ } -> true
-            | _ -> false)
-        in
-        let file_path = File_path.get_default_path_sig sg in
-        let base_ctxt =
-          Expansion_context.Base.top_level ~tool_name ~file_path ~input_name
-        in
-        let header, footer =
-          match enclose_intf with
-          | None -> ([], [])
-          | Some f ->
-              let whole_loc =
-                loc_of_list sg ~get_loc:(fun sg -> sg.Parsetree.psig_loc)
-              in
-              gen_header_and_footer Signature_item whole_loc (f base_ctxt)
-        in
-        map#signature base_ctxt (List.concat [ attrs; header; sg; footer ])
+      let attrs, sg =
+        List.split_while sg_with_attrs ~f:(function
+          | { psig_desc = Psig_attribute _; _ } -> true
+          | _ -> false)
       in
-      match intf with None -> sg | Some f -> f ctxt sg
+      let file_path = get_default_path_sig sg in
+      let base_ctxt =
+        Expansion_context.Base.top_level ~tool_name ~file_path ~input_name
+      in
+      let header, footer =
+        match enclose_intf with
+        | None -> ([], [])
+        | Some f ->
+            let whole_loc =
+              loc_of_list sg ~get_loc:(fun sg -> sg.Parsetree.psig_loc)
+            in
+            gen_header_and_footer Signature_item whole_loc (f base_ctxt)
+      in
+      map#signature base_ctxt (List.concat [ attrs; header; sg; footer ])
+      >>= fun sg -> match intf with None -> return sg | Some f -> f ctxt sg
     in
     { t with impl = Some map_impl; intf = Some map_intf }
 
@@ -386,7 +429,7 @@ let register_transformation ?extensions ?rules ?enclose_impl ?enclose_intf ?impl
 
 let register_code_transformation ~name ?(aliases = []) ~impl ~intf =
   register_transformation name ~impl ~intf ~aliases
-  [@@warning "-16"]
+[@@warning "-16"]
 (* This function triggers a warning 16 as of ocaml 4.12 *)
 
 let register_transformation_using_ocaml_current_ast ?impl ?intf =
@@ -406,13 +449,14 @@ let debug_dropped_attribute name ~old_dropped ~new_dropped =
       Printf.eprintf "The following attributes %s after applying %s:\n" what
         name;
       List.iter diff ~f:(fun { Location.txt; loc } ->
-          Caml.Format.eprintf "- %a: %s\n" Location.print loc txt);
-      Caml.Format.eprintf "@.")
+          Stdlib.Format.eprintf "- %a: %s\n" Location.print loc txt);
+      Stdlib.Format.eprintf "@.")
   in
   print_diff "disappeared" new_dropped old_dropped;
   print_diff "reappeared" old_dropped new_dropped
 
-let get_whole_ast_passes ~hook ~expect_mismatch_handler ~tool_name ~input_name =
+let get_whole_ast_passes ~embed_errors ~hook ~expect_mismatch_handler ~tool_name
+    ~input_name =
   let cts =
     match !apply_list with
     | None -> List.rev !Transform.all
@@ -430,18 +474,18 @@ let get_whole_ast_passes ~hook ~expect_mismatch_handler ~tool_name ~input_name =
   in
   (* Allow only one preprocessor to assure deterministic order *)
   (if List.length preprocess > 1 then
-   let pp =
-     String.concat ~sep:", " (List.map preprocess ~f:(fun t -> t.name))
-   in
-   let err =
-     Printf.sprintf "At most one preprocessor is allowed, while got: %s" pp
-   in
-   failwith err);
+     let pp =
+       String.concat ~sep:", " (List.map preprocess ~f:(fun t -> t.name))
+     in
+     let err =
+       Printf.sprintf "At most one preprocessor is allowed, while got: %s" pp
+     in
+     failwith err);
   let make_generic transforms =
     if !no_merge then
       List.map transforms
         ~f:
-          (Transform.merge_into_generic_mappers ~hook ~tool_name
+          (Transform.merge_into_generic_mappers ~embed_errors ~hook ~tool_name
              ~expect_mismatch_handler ~input_name)
     else
       (let get_enclosers ~f =
@@ -472,8 +516,8 @@ let get_whole_ast_passes ~hook ~expect_mismatch_handler ~tool_name ~input_name =
                      let footers = List.concat (List.rev footers) in
                      (headers, footers))
            in
-           Transform.builtin_of_context_free_rewriters ~rules ~hook
-             ~expect_mismatch_handler
+           Transform.builtin_of_context_free_rewriters ~rules ~embed_errors
+             ~hook ~expect_mismatch_handler
              ~enclose_impl:(merge_encloser impl_enclosers)
              ~enclose_intf:(merge_encloser intf_enclosers)
              ~tool_name ~input_name
@@ -483,61 +527,58 @@ let get_whole_ast_passes ~hook ~expect_mismatch_handler ~tool_name ~input_name =
   in
   linters @ preprocess @ before_instrs @ make_generic cts @ after_instrs
 
-let apply_transforms (type t) ~tool_name ~file_path ~field ~lint_field
-    ~dropped_so_far ~hook ~expect_mismatch_handler ~input_name ~f_exception
-    ~embed_errors x =
-  let exception
-    Wrapper of t list * label loc list * (location * label) list * exn
-  in
+let apply_transforms ~tool_name ~file_path ~field ~lint_field ~dropped_so_far
+    ~hook ~expect_mismatch_handler ~input_name ~embed_errors ast =
   let cts =
-    get_whole_ast_passes ~tool_name ~hook ~expect_mismatch_handler ~input_name
+    get_whole_ast_passes ~tool_name ~embed_errors ~hook ~expect_mismatch_handler
+      ~input_name
   in
-  let return (x, _dropped, lint_errors) =
-    ( x,
+  let finish (ast, _dropped, lint_errors, errors) =
+    ( ast,
       List.map lint_errors ~f:(fun (loc, s) ->
-          Common.attribute_of_warning loc s) )
+          Common.attribute_of_warning loc s),
+      errors )
   in
-  try
-    let acc =
-      List.fold_left cts ~init:(x, [], [])
-        ~f:(fun (x, dropped, (lint_errors : _ list)) (ct : Transform.t) ->
-          let input_name =
-            match input_name with
-            | Some input_name -> input_name
-            | None -> "_none_"
-          in
-          let ctxt =
-            Expansion_context.Base.top_level ~tool_name ~file_path ~input_name
-          in
-          let lint_errors =
-            match lint_field ct with
-            | None -> lint_errors
-            | Some f -> (
-                try lint_errors @ f ctxt x
-                with exn when embed_errors ->
-                  raise @@ Wrapper (x, dropped, lint_errors, exn))
-          in
-          match field ct with
-          | None -> (x, dropped, lint_errors)
-          | Some f ->
-              let x =
-                try f ctxt x
-                with exn when embed_errors ->
-                  raise @@ Wrapper (x, dropped, lint_errors, exn)
-              in
-              let dropped =
-                if !debug_attribute_drop then (
-                  let new_dropped = dropped_so_far x in
-                  debug_dropped_attribute ct.name ~old_dropped:dropped
-                    ~new_dropped;
-                  new_dropped)
-                else []
-              in
-              (x, dropped, lint_errors))
-    in
-    Ok (return acc)
-  with Wrapper (x, dropped, lint_errors, exn) ->
-    Error (return (f_exception exn :: x, dropped, lint_errors))
+  let acc =
+    List.fold_left cts ~init:(ast, [], [], [])
+      ~f:(fun
+          (ast, dropped, (lint_errors : _ list), errors) (ct : Transform.t) ->
+        let input_name =
+          match input_name with
+          | Some input_name -> input_name
+          | None -> "_none_"
+        in
+        let ctxt =
+          Expansion_context.Base.top_level ~tool_name ~file_path ~input_name
+        in
+
+        let lint_errors, errors =
+          match lint_field ct with
+          | None -> (lint_errors, errors)
+          | Some f -> (
+              try (lint_errors @ f ctxt ast, errors)
+              with exn when embed_errors ->
+                (lint_errors, exn_to_loc_error exn :: errors))
+        in
+        match field ct with
+        | None -> (ast, dropped, lint_errors, errors)
+        | Some f ->
+            let (ast, more_errors), errors =
+              try (f ctxt ast, errors)
+              with exn when embed_errors ->
+                ((ast, []), exn_to_loc_error exn :: errors)
+            in
+            let dropped =
+              if !debug_attribute_drop then (
+                let new_dropped = dropped_so_far ast in
+                debug_dropped_attribute ct.name ~old_dropped:dropped
+                  ~new_dropped;
+                new_dropped)
+              else []
+            in
+            (ast, dropped, lint_errors, errors @ more_errors))
+  in
+  finish acc
 
 (*$*)
 
@@ -546,11 +587,6 @@ let error_to_str_extension error =
   let ext = Location.Error.to_extension error in
   Ast_builder.Default.pstr_extension ~loc ext []
 
-let exn_to_str_extension exn =
-  match Location.Error.of_exn exn with
-  | None -> raise exn
-  | Some error -> error_to_str_extension error
-
 (*$ str_to_sig _last_text_block *)
 
 let error_to_sig_extension error =
@@ -558,11 +594,6 @@ let error_to_sig_extension error =
   let ext = Location.Error.to_extension error in
   Ast_builder.Default.psig_extension ~loc ext []
 
-let exn_to_sig_extension exn =
-  match Location.Error.of_exn exn with
-  | None -> raise exn
-  | Some error -> error_to_sig_extension error
-
 (*$*)
 
 let error_to_extension error ~(kind : Kind.t) =
@@ -571,9 +602,7 @@ let error_to_extension error ~(kind : Kind.t) =
   | Impl -> Intf_or_impl.Impl [ error_to_str_extension error ]
 
 let exn_to_extension exn ~(kind : Kind.t) =
-  match Location.Error.of_exn exn with
-  | None -> raise exn
-  | Some error -> error_to_extension error ~kind
+  exn_to_loc_error exn |> error_to_extension ~kind
 
 (* +-----------------------------------------------------------------+
    | Actual rewriting of structure/signatures                        |
@@ -581,10 +610,11 @@ let exn_to_extension exn ~(kind : Kind.t) =
 
 let print_passes () =
   let tool_name = "ppxlib_driver" in
+  let embed_errors = false in
   let hook = Context_free.Generated_code_hook.nop in
   let expect_mismatch_handler = Context_free.Expect_mismatch_handler.nop in
   let cts =
-    get_whole_ast_passes ~hook ~expect_mismatch_handler ~tool_name
+    get_whole_ast_passes ~embed_errors ~hook ~expect_mismatch_handler ~tool_name
       ~input_name:None
   in
   if !perform_checks then
@@ -595,6 +625,12 @@ let print_passes () =
     if !perform_checks_on_extensions then
       Printf.printf "<builtin:check-unused-extensions>\n")
 
+let sort_errors_by_loc errors =
+  List.sort errors ~cmp:(fun error error' ->
+      let loc = Location.Error.get_location error in
+      let loc' = Location.Error.get_location error' in
+      Location.compare loc loc')
+
 (*$*)
 
 let map_structure_gen st ~tool_name ~hook ~expect_mismatch_handler ~input_name
@@ -615,6 +651,16 @@ let map_structure_gen st ~tool_name ~hook ~expect_mismatch_handler ~input_name
     in
     st
   in
+  let with_errors errors st =
+    let sorted = sort_errors_by_loc errors in
+    List.map sorted ~f:(fun error ->
+        Ast_builder.Default.pstr_extension
+          ~loc:(Location.Error.get_location error)
+          (Location.Error.to_extension error)
+          []
+        |> ghost#structure_item)
+    @ st
+  in
   let cookies_and_check st =
     Cookies.call_post_handlers T;
     let errors =
@@ -631,34 +677,25 @@ let map_structure_gen st ~tool_name ~hook ~expect_mismatch_handler ~input_name
         in
         let not_seen_errors = Attribute.collect_unseen_errors () in
         (if !perform_locations_check then
-         let open Location_check in
-         ignore
-           ((enforce_invariants !loc_fname)#structure st
-              Non_intersecting_ranges.empty
-             : Non_intersecting_ranges.t));
-        let errors =
-          unused_attributes_errors @ unused_extension_errors @ not_seen_errors
-        in
-        errors
-        |> List.map ~f:Location.Error.to_extension
-        |> List.map ~f:(fun e ->
-               Ast_builder.Default.pstr_extension ~loc:Location.none e []))
+           let open Location_check in
+           ignore
+             ((enforce_invariants !loc_fname)#structure st
+                Non_intersecting_ranges.empty
+               : Non_intersecting_ranges.t));
+        unused_attributes_errors @ unused_extension_errors @ not_seen_errors)
       else []
     in
-    errors @ st
+    with_errors errors st
   in
-  let file_path = File_path.get_default_path_str st in
-  match
+  let file_path = get_default_path_str st in
+  let st, lint_errors, errors =
     apply_transforms st ~tool_name ~file_path
       ~field:(fun (ct : Transform.t) -> ct.impl)
       ~lint_field:(fun (ct : Transform.t) -> ct.lint_impl)
       ~dropped_so_far:Attribute.dropped_so_far_structure ~hook
-      ~expect_mismatch_handler ~input_name
-      ~f_exception:(fun exn -> exn_to_str_extension exn)
-      ~embed_errors
-  with
-  | Error (st, lint_errors) -> Error (lint lint_errors st)
-  | Ok (st, lint_errors) -> Ok (st |> lint lint_errors |> cookies_and_check)
+      ~expect_mismatch_handler ~input_name ~embed_errors
+  in
+  st |> lint lint_errors |> cookies_and_check |> with_errors (List.rev errors)
 
 let map_structure st =
   match
@@ -668,7 +705,7 @@ let map_structure st =
       ~expect_mismatch_handler:Context_free.Expect_mismatch_handler.nop
       ~input_name:None ~embed_errors:false
   with
-  | Ok ast | Error ast -> ast
+  | ast -> ast
 
 (*$ str_to_sig _last_text_block *)
 
@@ -690,6 +727,16 @@ let map_signature_gen sg ~tool_name ~hook ~expect_mismatch_handler ~input_name
     in
     sg
   in
+  let with_errors errors sg =
+    let sorted = sort_errors_by_loc errors in
+    List.map sorted ~f:(fun error ->
+        Ast_builder.Default.psig_extension
+          ~loc:(Location.Error.get_location error)
+          (Location.Error.to_extension error)
+          []
+        |> ghost#signature_item)
+    @ sg
+  in
   let cookies_and_check sg =
     Cookies.call_post_handlers T;
     let errors =
@@ -706,34 +753,25 @@ let map_signature_gen sg ~tool_name ~hook ~expect_mismatch_handler ~input_name
         in
         let not_seen_errors = Attribute.collect_unseen_errors () in
         (if !perform_locations_check then
-         let open Location_check in
-         ignore
-           ((enforce_invariants !loc_fname)#signature sg
-              Non_intersecting_ranges.empty
-             : Non_intersecting_ranges.t));
-        let errors =
-          unused_attributes_errors @ unused_extension_errors @ not_seen_errors
-        in
-        errors
-        |> List.map ~f:Location.Error.to_extension
-        |> List.map ~f:(fun e ->
-               Ast_builder.Default.psig_extension ~loc:Location.none e []))
+           let open Location_check in
+           ignore
+             ((enforce_invariants !loc_fname)#signature sg
+                Non_intersecting_ranges.empty
+               : Non_intersecting_ranges.t));
+        unused_attributes_errors @ unused_extension_errors @ not_seen_errors)
       else []
     in
-    errors @ sg
+    with_errors errors sg
   in
-  let file_path = File_path.get_default_path_sig sg in
-  match
+  let file_path = get_default_path_sig sg in
+  let sg, lint_errors, errors =
     apply_transforms sg ~tool_name ~file_path
       ~field:(fun (ct : Transform.t) -> ct.intf)
       ~lint_field:(fun (ct : Transform.t) -> ct.lint_intf)
       ~dropped_so_far:Attribute.dropped_so_far_signature ~hook
-      ~expect_mismatch_handler ~input_name
-      ~f_exception:(fun exn -> exn_to_sig_extension exn)
-      ~embed_errors
-  with
-  | Error (sg, lint_errors) -> Error (lint lint_errors sg)
-  | Ok (sg, lint_errors) -> Ok (sg |> lint lint_errors |> cookies_and_check)
+      ~expect_mismatch_handler ~input_name ~embed_errors
+  in
+  sg |> lint lint_errors |> cookies_and_check |> with_errors (List.rev errors)
 
 let map_signature sg =
   match
@@ -743,7 +781,7 @@ let map_signature sg =
       ~expect_mismatch_handler:Context_free.Expect_mismatch_handler.nop
       ~input_name:None ~embed_errors:false
   with
-  | Ok ast | Error ast -> ast
+  | ast -> ast
 
 (*$*)
 
@@ -764,7 +802,7 @@ let versioned_errorf input_version input_file_name =
       in
       Error (err, input_version))
 
-let remove_no_error fn = try Caml.Sys.remove fn with Sys_error _ -> ()
+let remove_no_error fn = try Stdlib.Sys.remove fn with Sys_error _ -> ()
 
 let protectx x ~f ~finally =
   match f x with
@@ -779,7 +817,7 @@ let with_preprocessed_file fn ~f =
   match !preprocessor with
   | None -> f fn
   | Some pp ->
-      protectx (Caml.Filename.temp_file "ocamlpp" "") ~finally:remove_no_error
+      protectx (Stdlib.Filename.temp_file "ocamlpp" "") ~finally:remove_no_error
         ~f:(fun tmpfile ->
           match System.run_preprocessor ~pp ~input:fn ~output:tmpfile with
           | Ok () -> f tmpfile
@@ -985,10 +1023,11 @@ module File_property = struct
             Some (t.name, t.sexp_of_t v))
 end
 
-module Create_file_property (Name : sig
-  val name : string
-end)
-(T : Sexpable.S) =
+module Create_file_property
+    (Name : sig
+      val name : string
+    end)
+    (T : Sexpable.S) =
 struct
   let t : _ File_property.t =
     { name = Name.name; data = None; sexp_of_t = T.sexp_of_t }
@@ -1006,7 +1045,7 @@ let process_ast (ast : Intf_or_impl.t) ~input_name ~tool_name ~hook
           map_signature_gen x ~tool_name ~hook ~expect_mismatch_handler
             ~input_name:(Some input_name) ~embed_errors
         with
-        | Error ast | Ok ast -> ast
+        | ast -> ast
       in
       Intf_or_impl.Intf ast
   | Impl x ->
@@ -1015,7 +1054,7 @@ let process_ast (ast : Intf_or_impl.t) ~input_name ~tool_name ~hook
           map_structure_gen x ~tool_name ~hook ~expect_mismatch_handler
             ~input_name:(Some input_name) ~embed_errors
         with
-        | Error ast | Ok ast -> ast
+        | ast -> ast
       in
       Intf_or_impl.Impl ast
 
@@ -1083,7 +1122,7 @@ let process_file (kind : Kind.t) fn ~input_name ~relocate ~output_mode
   let mismatches_found =
     match !corrections with
     | [] ->
-        if Caml.Sys.file_exists corrected then Caml.Sys.remove corrected;
+        if Stdlib.Sys.file_exists corrected then Stdlib.Sys.remove corrected;
         false
     | corrections ->
         Reconcile.reconcile corrections
@@ -1097,14 +1136,14 @@ let process_file (kind : Kind.t) fn ~input_name ~relocate ~output_mode
   | Null -> ()
   | Pretty_print ->
       with_output output ~binary:false ~f:(fun oc ->
-          let ppf = Caml.Format.formatter_of_out_channel oc in
+          let ppf = Stdlib.Format.formatter_of_out_channel oc in
           (match ast with
           | Intf ast -> Pprintast.signature ppf ast
           | Impl ast -> Pprintast.structure ppf ast);
           let null_ast =
             match ast with Intf [] | Impl [] -> true | _ -> false
           in
-          if not null_ast then Caml.Format.pp_print_newline ppf ())
+          if not null_ast then Stdlib.Format.pp_print_newline ppf ())
   | Dump_ast ->
       with_output output ~binary:true ~f:(fun oc ->
           Ast_io.write oc
@@ -1112,12 +1151,12 @@ let process_file (kind : Kind.t) fn ~input_name ~relocate ~output_mode
             ~add_ppx_context:true)
   | Dparsetree ->
       with_output output ~binary:false ~f:(fun oc ->
-          let ppf = Caml.Format.formatter_of_out_channel oc in
+          let ppf = Stdlib.Format.formatter_of_out_channel oc in
           let ast = add_cookies ast in
           (match ast with
           | Intf ast -> Sexp.pp_hum ppf (Ast_traverse.sexp_of#signature ast)
           | Impl ast -> Sexp.pp_hum ppf (Ast_traverse.sexp_of#structure ast));
-          Caml.Format.pp_print_newline ppf ())
+          Stdlib.Format.pp_print_newline ppf ())
   | Reconcile mode ->
       Reconcile.reconcile !replacements
         ~contents:(Lazy.force input_contents)
@@ -1129,7 +1168,7 @@ let process_file (kind : Kind.t) fn ~input_name ~relocate ~output_mode
   then (
     Ppxlib_print_diff.print () ~file1:fn ~file2:corrected ~use_color:!use_color
       ?diff_command:!diff_command;
-    Caml.exit 1)
+    Stdlib.exit 1)
 
 let output_mode = ref Pretty_print
 let output = ref None
@@ -1183,7 +1222,7 @@ let parse_apply_list s =
                Transform.has_name ct name))
       then
         raise
-          (Caml.Arg.Bad
+          (Stdlib.Arg.Bad
              (Printf.sprintf "code transformation '%s' does not exist" name)));
   names
 
@@ -1312,7 +1351,7 @@ let standalone_args =
       " Print the parsetree (same as ocamlc -dparsetree)" );
     ( "-embed-errors",
       Arg.Set embed_errors,
-      " Embed errors in the output AST (default: true when -dump-ast, false \
+      " Embed errors in the output AST (default: true when -as-pp, false \
        otherwise)" );
     ( "-null",
       Arg.Unit (fun () -> set_output_mode Null),
@@ -1382,14 +1421,14 @@ let standalone_main () =
   interpret_mask ();
   if !request_print_transformations then (
     print_transformations ();
-    Caml.exit 0);
+    Stdlib.exit 0);
   if !request_print_passes then (
     print_passes ();
-    Caml.exit 0);
+    Stdlib.exit 0);
   match !input with
   | None ->
       Printf.eprintf "%s: no input file given\n%!" exe_name;
-      Caml.exit 2
+      Stdlib.exit 2
   | Some fn ->
       let kind =
         match !kind with
@@ -1401,7 +1440,7 @@ let standalone_main () =
                 Printf.eprintf
                   "%s: don't know what to do with '%s', use -impl or -intf.\n"
                   exe_name fn;
-                Caml.exit 2)
+                Stdlib.exit 2)
       in
       let input_name, relocate =
         match !loc_fname with None -> (fn, false) | Some fn -> (fn, true)
@@ -1432,10 +1471,10 @@ let parse_input passed_in_args ~valid_args ~incorrect_input_msg =
   with
   | Arg.Bad msg ->
       Printf.eprintf "%s" msg;
-      Caml.exit 2
+      Stdlib.exit 2
   | Arg.Help msg ->
       Printf.eprintf "%s" msg;
-      Caml.exit 0
+      Stdlib.exit 0
 
 let run_as_ppx_rewriter_main ~standalone_args ~usage input =
   let valid_args = get_args ~standalone_args () in
@@ -1446,23 +1485,23 @@ let run_as_ppx_rewriter_main ~standalone_args ~usage input =
       parse_input prog_name_and_flags ~valid_args ~incorrect_input_msg:usage;
       interpret_mask ();
       rewrite_binary_ast_file input_fn output_fn;
-      Caml.exit 0
+      Stdlib.exit 0
   | [ help; _ ] when String.equal help "-help" || String.equal help "--help" ->
       parse_input input ~valid_args ~incorrect_input_msg:usage;
       assert false
   | _ ->
       Printf.eprintf "Usage: %s\n%!" usage;
-      Caml.exit 2
+      Stdlib.exit 2
 
 let standalone_run_as_ppx_rewriter () =
-  let n = Array.length Caml.Sys.argv in
+  let n = Array.length Stdlib.Sys.argv in
   let usage =
     Printf.sprintf "%s -as-ppx [extra_args] <infile> <outfile>" exe_name
   in
   let argv = Array.make (n - 1) "" in
-  argv.(0) <- Caml.Sys.argv.(0);
+  argv.(0) <- Stdlib.Sys.argv.(0);
   for i = 1 to n - 2 do
-    argv.(i) <- Caml.Sys.argv.(i + 1)
+    argv.(i) <- Stdlib.Sys.argv.(i + 1)
   done;
   let standalone_args =
     List.map standalone_args ~f:(fun (arg, spec, _doc) ->
@@ -1474,23 +1513,25 @@ let standalone () =
   Astlib.init_error_reporting_style_using_env_vars ();
   try
     if
-      Array.length Caml.Sys.argv >= 2
+      Array.length Stdlib.Sys.argv >= 2
       &&
-      match Caml.Sys.argv.(1) with "-as-ppx" | "--as-ppx" -> true | _ -> false
+      match Stdlib.Sys.argv.(1) with
+      | "-as-ppx" | "--as-ppx" -> true
+      | _ -> false
     then standalone_run_as_ppx_rewriter ()
     else standalone_main ();
-    Caml.exit 0
+    Stdlib.exit 0
   with exn ->
-    Location.report_exception Caml.Format.err_formatter exn;
-    Caml.exit 1
+    Location.report_exception Stdlib.Format.err_formatter exn;
+    Stdlib.exit 1
 
 let run_as_ppx_rewriter () =
   let usage = Printf.sprintf "%s [extra_args] <infile> <outfile>" exe_name in
-  let input = Caml.Sys.argv in
+  let input = Stdlib.Sys.argv in
   try run_as_ppx_rewriter_main ~standalone_args:[] ~usage input
   with exn ->
-    Location.report_exception Caml.Format.err_formatter exn;
-    Caml.exit 1
+    Location.report_exception Stdlib.Format.err_formatter exn;
+    Stdlib.exit 1
 
 let pretty () = !pretty
 
