@@ -24,7 +24,6 @@ let init lexbuf fname =
     { pos_fname = fname; pos_lnum = 1; pos_bol = 0; pos_cnum = 0 }
 
 let raise_errorf ?loc fmt = L.raise_errorf ?loc fmt
-
 let report_exception = L.report_exception
 
 let of_lexbuf (lexbuf : Lexing.lexbuf) =
@@ -35,7 +34,7 @@ let of_lexbuf (lexbuf : Lexing.lexbuf) =
   }
 
 let print ppf t =
-  Caml.Format.fprintf ppf "File \"%s\", line %d, characters %d-%d:"
+  Stdlib.Format.fprintf ppf "File \"%s\", line %d, characters %d-%d:"
     t.loc_start.pos_fname t.loc_start.pos_lnum
     (t.loc_start.pos_cnum - t.loc_start.pos_bol)
     (t.loc_end.pos_cnum - t.loc_start.pos_bol)
@@ -56,7 +55,6 @@ let compare_pos p1 p2 =
   | n -> n
 
 let min_pos p1 p2 = if compare_pos p1 p2 <= 0 then p1 else p2
-
 let max_pos p1 p2 = if compare_pos p1 p2 >= 0 then p1 else p2
 
 let compare loc1 loc2 =
@@ -70,9 +68,14 @@ module Error = struct
   let createf ~loc fmt = Format.kasprintf (fun str -> make ~loc ~sub:[] str) fmt
 end
 
+let error_extensionf ~loc fmt =
+  Format.kasprintf
+    (fun str -> Error.to_extension @@ Error.make ~loc ~sub:[] str)
+    fmt
+
 exception Error = L.Error
 
 let () =
-  Caml.Printexc.register_printer (function
+  Stdlib.Printexc.register_printer (function
     | Error e -> Some (Error.message e)
     | _ -> None)
